/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This software is Copyright � 2011 Yuri Gonzaga Gon�alves da Costa <yuriggc at gmail>, and it is hereby released to the general public under the following terms:*
 * Redistribution and use in source and binary forms, with or without modification, are permitted.                                                                 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package bcrypt_java;

import Utils.Utils;
import java.io.UnsupportedEncodingException;


public class BCrypt {

    private Eksblowfish eks;

    public BCrypt() {
        this.eks = new Eksblowfish();
    }

    public long[] cript(int cost, long salt[], byte pwd[]) {
        eks.setup(cost, salt, pwd);
        //ctext = "OrpheanBeholderScryDoubt";
        long ctext[] = new long[3];
        ctext[0] = 5724761670021115458L;
        ctext[1] = 7307212906740150867L;
        ctext[2] = 7165923291934581364L;
        for (int i = 0; i < 64; i++) {
            ctext[0] = eks.cryptBF(ctext[0]);
            ctext[1] = eks.cryptBF(ctext[1]);
            ctext[2] = eks.cryptBF(ctext[2]);
        }
        return ctext;
    }

        /**
     * Converts the byte array salt to a long array and calls appropriate crypt.
     * @param cost
     * @param salt
     * @param pwd
     * @return
     */
    public long[] cript(int cost, byte salt[], byte pwd[]) {

        long s[] = {0L, 0L};
        for (int i = 15; i >= 8; i--) {
            s[0] = (s[0] << 8) | (salt[15 - i] & 0xFFL);
        }

        for (int i = 7; i >= 0; i--) {
            s[1] = (s[1] << 8) | (salt[15 - i] & 0xFF);
        }

        return cript(cost, s, pwd);
    }

    public static String hashpw(String password, String salt) {
        BCrypt B;
        String real_salt;
        long[] hashedLong;
        byte passwordb[], saltb[], hashed[];
        char minor = (char) 0;
        int cost, off = 0;
        StringBuilder rs = new StringBuilder();

        if (salt.charAt(0) != '$' || salt.charAt(1) != '2') {
            throw new IllegalArgumentException("Invalid salt version");
        }
        if (salt.charAt(2) == '$') {
            off = 3;
        } else {
            minor = salt.charAt(2);
            if (minor != 'a' || salt.charAt(3) != '$') {
                throw new IllegalArgumentException("Invalid salt revision");
            }
            off = 4;
        }

        // Extract number of rounds
        if (salt.charAt(off + 2) > '$') {
            throw new IllegalArgumentException("Missing salt rounds");
        }
        cost = Integer.parseInt(salt.substring(off, off + 2));
        System.out.printf("\ncost %d\n", cost);
        real_salt = salt.substring(off + 3, off + 25);
        // System.out.printf("real_salt %s\n", real_salt);

        try {
            passwordb = (password + (minor >= 'a' ? "\000" : "")).getBytes("UTF-8");
        } catch (UnsupportedEncodingException uee) {
            throw new AssertionError("UTF-8 is not supported");
        }

        System.out.print("password: ");
        for (int i = 0; i < passwordb.length; i++) {
            byte b = passwordb[i];
            System.out.printf(" %d", b);
        }
        System.out.println("");
        saltb = Utils.decode_base64(real_salt, Utils.BCRYPT_SALT_LEN);
        System.out.print("salt: ");
        for (int i = 0; i < saltb.length; i++) {
            byte b = saltb[i];
            System.out.printf("%02x", b);
        }
        System.out.println("");
        B = new BCrypt();

        hashedLong = B.cript(cost, saltb, passwordb);
        hashed = Utils.vectorLong2Bytes(hashedLong);

        for (int i = 0; i < hashed.length; i++) {
            System.out.printf("%02x", hashed[i]);
        }

        rs.append("$2");
        if (minor >= 'a') {
            rs.append(minor);
        }
        rs.append("$");
        if (cost < 10) {
            rs.append("0");
        }
        rs.append(Integer.toString(cost));
        rs.append("$");
        rs.append(Utils.encode_base64(saltb, saltb.length));
        rs.append(Utils.encode_base64(hashed, 23));
        return rs.toString();
    }

    public static void main(String[] args) {

        /********* Tests **********/
        String test_vectors[][] = {
            {   "password",
                "$2a$05$bvIG6Nmid91Mu9RcmmWZfO",
                "$2a$05$bvIG6Nmid91Mu9RcmmWZfO5HJIMCT8riNW0hEp8f6/FuA2/mHZFpe"
            }
        };

        System.out.print("BCrypt.hashpw(): ");
        String plain = test_vectors[0][0];
        String use_salt = test_vectors[0][1];
        String expected = test_vectors[0][2];

        String hashed = BCrypt.hashpw(plain, use_salt);
        System.out.println("\n");
        System.out.println(hashed);
        if(hashed.compareTo(expected)==0){
            System.out.println("Ok");
        }else{
            System.out.println("Not Ok");
        }
    }

}
